import createExpress from 'express';
import * as HTTP from 'http';
import stripAnsi from 'strip-ansi';
import { noop } from '../../lib/utils';
import * as DevMode from '../dev-mode';
import { assembledGuard } from '../utils';
import { createRequestHandlerGraphQL } from './handler-graphql';
import { createRequestHandlerPlayground } from './handler-playground';
import { log } from './logger';
import { createServerSettingsManager } from './settings';
const resolverLogger = log.child('graphql');
export const defaultState = {
    running: false,
    httpServer: HTTP.createServer(),
    createContext: null,
};
export function create(appState) {
    const settings = createServerSettingsManager();
    const express = createExpress();
    const state = Object.assign({}, defaultState);
    const api = {
        express,
        handlers: {
            get playground() {
                var _a;
                return ((_a = assembledGuard(appState, 'app.server.handlers.playground', () => {
                    // todo should be accessing settings from assembled app state settings
                    return wrapHandlerWithErrorHandling(createRequestHandlerPlayground({ graphqlEndpoint: settings.data.path }));
                })) !== null && _a !== void 0 ? _a : noop);
            },
            get graphql() {
                var _a;
                return ((_a = assembledGuard(appState, 'app.server.handlers.graphql', () => {
                    return wrapHandlerWithErrorHandling(createRequestHandlerGraphQL(appState.assembled.schema, appState.assembled.createContext));
                })) !== null && _a !== void 0 ? _a : noop);
            },
        },
    };
    const internalServer = {
        private: {
            settings,
            state,
            reset() {
                internalServer.private.state = Object.assign({}, defaultState);
            },
            assemble(loadedRuntimePlugins, schema) {
                state.httpServer.on('request', express);
                if (settings.data.playground) {
                    express.get(settings.data.playground.path, wrapHandlerWithErrorHandling(createRequestHandlerPlayground({ graphqlEndpoint: settings.data.path })));
                }
                const createContext = createContextCreator(appState.schemaComponent.contextContributors, loadedRuntimePlugins);
                const graphqlHandler = createRequestHandlerGraphQL(schema, createContext);
                express.post(settings.data.path, wrapHandlerWithErrorHandling(graphqlHandler));
                express.get(settings.data.path, wrapHandlerWithErrorHandling(graphqlHandler));
                return { createContext };
            },
            async start() {
                await httpListen(state.httpServer, { port: settings.data.port, host: settings.data.host });
                state.running = true;
                // About !
                // 1. We do not support listening on unix domain sockets so string
                //    value will never be present here.
                // 2. We are working within the listen callback so address will not be null
                const address = state.httpServer.address();
                settings.data.startMessage({
                    port: address.port,
                    host: address.address,
                    ip: address.address,
                    path: settings.data.path,
                    playgroundPath: settings.data.playground ? settings.data.playground.path : undefined,
                });
                DevMode.sendServerReadySignalToDevModeMaster();
            },
            async stop() {
                if (!state.running) {
                    log.warn('You called `server.stop` but the server was not running.');
                    return Promise.resolve();
                }
                await httpClose(state.httpServer);
                state.running = false;
            },
        },
        public: api,
    };
    return internalServer;
}
/**
 * Log http errors during development.
 */
const wrapHandlerWithErrorHandling = (handler) => {
    return async (req, res) => {
        var _a;
        await handler(req, res);
        if (res.statusCode !== 200 && res.error) {
            const error = res.error;
            const colorlessMessage = stripAnsi(error.message);
            if (process.env.NEXUS_STAGE === 'dev') {
                resolverLogger.error((_a = error.stack) !== null && _a !== void 0 ? _a : error.message);
            }
            else {
                resolverLogger.error('An exception occured in one of your resolver', {
                    error: error.stack ? stripAnsi(error.stack) : colorlessMessage,
                });
            }
            // todo bring back payload sanitization for data sent to clients
            // error.message = colorlessMessage
        }
    };
};
/**
 * Combine all the context contributions defined in the app and in plugins.
 */
function createContextCreator(contextContributors, plugins) {
    const createContext = async (req) => {
        let context = {};
        // Integrate context from plugins
        for (const plugin of plugins) {
            if (!plugin.context)
                continue;
            const contextContribution = await plugin.context.create(req);
            Object.assign(context, contextContribution);
        }
        // Integrate context from app context api
        // TODO good runtime feedback to user if something goes wrong
        for (const contextContributor of contextContributors) {
            const contextContribution = await contextContributor(req);
            Object.assign(context, contextContribution);
        }
        Object.assign(context, { log: log.child('request') });
        return context;
    };
    return createContext;
}
function httpListen(server, options) {
    return new Promise((res, rej) => {
        server.listen(options, () => {
            res();
        });
    });
}
function httpClose(server) {
    return new Promise((res, rej) => {
        server.close((err) => {
            if (err) {
                rej(err);
            }
            else {
                res();
            }
        });
    });
}
//# sourceMappingURL=server.js.map